@@ -5,7 +5,7 @@
 %%% Created : 12 Dec 2004 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2013   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2014   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -17,10 +17,9 @@
 %%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 %%% General Public License for more details.
 %%%
-%%% You should have received a copy of the GNU General Public License
-%%% along with this program; if not, write to the Free Software
-%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-%%% 02111-1307 USA
+%%% You should have received a copy of the GNU General Public License along
+%%% with this program; if not, write to the Free Software Foundation, Inc.,
+%%% 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 %%%
 %%%----------------------------------------------------------------------
 
@@ -55,10 +54,8 @@ start(Host) ->
             end,
             "extauth"),
     extauth:start(Host, Cmd),
-    case check_cache_last_options(Host) of
-      cache -> ok = ejabberd_auth_internal:start(Host);
-      no_cache -> ok
-    end.
+    check_cache_last_options(Host),
+    ejabberd_auth_internal:start(Host).
 
 check_cache_last_options(Server) ->
     case get_cache_option(Server) of
@@ -173,7 +170,9 @@ remove_user(User, Server, Password) ->
 get_cache_option(Host) ->
     case ejabberd_config:get_option(
            {extauth_cache, Host},
-           fun(I) when is_integer(I), I > 0 -> I end) of
+           fun(false) -> undefined;
+              (I) when is_integer(I), I >= 0 -> I
+           end) of
         undefined -> false;
         CacheTime -> {true, CacheTime}
     end.
