@@ -345,12 +345,18 @@ build_fault_response(Code, ParseString, ParseArgs) ->
 do_command(AccessCommands, Auth, Command, AttrL, ArgsF,
 	   ResultF) ->
     ArgsFormatted = format_args(AttrL, ArgsF),
+    AuthBin = convert_auth(Auth),
     Result =
-	ejabberd_commands:execute_command(AccessCommands, Auth,
+	ejabberd_commands:execute_command(AccessCommands, AuthBin,
 					  Command, ArgsFormatted),
     ResultFormatted = format_result(Result, ResultF),
     {command_result, ResultFormatted}.
 
+convert_auth(noauth) ->
+    noauth;
+convert_auth({UserT, ServerT, PasswordT}) ->
+    {list_to_binary(UserT), list_to_binary(ServerT), list_to_binary(PasswordT)}.
+
 %%-----------------------------
 %% Format arguments
 %%-----------------------------
@@ -422,7 +428,11 @@ format_arg({array, Elements}, {list, ElementsDef})
 format_arg(Arg, integer) when is_integer(Arg) -> Arg;
 format_arg(Arg, binary) when is_list(Arg) -> list_to_binary(Arg);
 format_arg(Arg, binary) when is_binary(Arg) -> Arg;
-format_arg(Arg, string) when is_binary(Arg) -> Arg.
+format_arg(Arg, string) when is_list(Arg) -> list_to_binary(Arg);
+format_arg(Arg, string) when is_binary(Arg) -> Arg;
+format_arg(Arg, Format) ->
+    ?ERROR_MSG("don't know how to format Arg ~p for format ~p", [Arg, Format]),
+    throw({error_formatting_argument, Arg, Format}).
 
 %% -----------------------------
 %% Result
